@@ -86,6 +86,8 @@ class BuildManager:
             self.project_root, self.src_dir, exclude_patterns=exclude_patterns
         )
         self.subfolder_config: SubfolderBuildConfig | None = None
+        # Cache for package name lookups (expensive operation)
+        self._packages_distributions_cache: dict[str, list[str]] | None = None
 
         # Check if it's a valid Python package directory
         if not any(self.src_dir.glob("*.py")) and not (self.src_dir / "__init__.py").exists():
@@ -268,6 +270,21 @@ class BuildManager:
         for dep in external_deps:
             self._copy_dependency(dep)
 
+        # For subfolder builds, extract third-party dependencies and add to pyproject.toml
+        if self._is_subfolder_build() and self.subfolder_config:
+            # Re-analyze all Python files (including copied dependencies) to find third-party imports
+            print("Analyzing Python files for third-party dependencies...")
+            all_python_files = analyzer.find_all_python_files(self.src_dir)
+            print(f"Found {len(all_python_files)} Python files to analyze")
+            third_party_deps = self._extract_third_party_dependencies(all_python_files, analyzer)
+            if third_party_deps:
+                print(
+                    f"Found {len(third_party_deps)} third-party dependencies: {', '.join(third_party_deps)}"
+                )
+                self.subfolder_config.add_third_party_dependencies(third_party_deps)
+            else:
+                print("No third-party dependencies found in subfolder code")
+
         # Report ambiguous imports
         self._report_ambiguous_imports(python_files)
 
@@ -381,6 +398,192 @@ class BuildManager:
             elif src_item.is_dir():
                 self._copytree_excluding(src_item, dst_item)
 
+    def _get_package_name_from_import(self, module_name: str) -> str | None:
+        """
+        Get the actual PyPI package name from an import module name.
+
+        This handles cases where the import name differs from the package name
+        (e.g., 'import fitz' from 'pymupdf' package).
+
+        Args:
+            module_name: The module name from the import statement
+
+        Returns:
+            The actual package name, or None if not found
+        """
+        root_module = module_name.split(".")[0]
+        try:
+            # Try Python 3.10+ first (has packages_distributions)
+            import importlib.metadata as importlib_metadata
+
+            # Use packages_distributions() if available (Python 3.10+)
+            # Cache the result since it's expensive to call
+            if hasattr(importlib_metadata, "packages_distributions"):
+                if self._packages_distributions_cache is None:
+                    # Cache the packages_distributions() result
+                    self._packages_distributions_cache = importlib_metadata.packages_distributions()
+                packages_map = self._packages_distributions_cache
+                # packages_map is a dict mapping module names to list of distribution names
+                if root_module in packages_map:
+                    # Return the first distribution name (usually there's only one)
+                    dist_names = packages_map[root_module]
+                    if dist_names:
+                        return dist_names[0]
+
+            # Fallback: search all distributions (this can be slow, so limit search)
+            # Only check top-level package matches to speed up search
+            dist_count = 0
+            max_distributions_to_check = 1000  # Limit to prevent excessive searching
+            for dist in importlib_metadata.distributions():
+                dist_count += 1
+                if dist_count > max_distributions_to_check:
+                    # Too many distributions, give up to avoid hanging
+                    break
+                try:
+                    # Check distribution name first (fast check)
+                    dist_name = dist.metadata.get("Name", "")
+                    # If distribution name matches or contains the module name, check files
+                    if dist_name.lower().replace(
+                        "-", "_"
+                    ) == root_module.lower() or root_module.lower() in dist_name.lower().replace(
+                        "-", "_"
+                    ):
+                        # Check if this distribution provides the module by looking at its files
+                        files = dist.files or []
+                        # Limit file checking to first 100 files per distribution
+                        file_count = 0
+                        for file in files:
+                            file_count += 1
+                            if file_count > 100:
+                                break
+                            file_str = str(file)
+                            # Check if file is the module itself or in a package directory
+                            if (
+                                file.suffix == ".py"
+                                and (file.stem == root_module or file.stem == "__init__")
+                            ) or (
+                                "/" in file_str
+                                and (
+                                    file_str.startswith(f"{root_module}/")
+                                    or file_str.startswith(f"{root_module.replace('_', '-')}/")
+                                )
+                            ):
+                                return dist.metadata["Name"]
+                except Exception:
+                    continue
+
+        except ImportError:
+            try:
+                # Fallback for older Python versions
+                import importlib_metadata
+
+                # Search all distributions
+                for dist in importlib_metadata.distributions():
+                    try:
+                        files = dist.files or []
+                        for file in files:
+                            file_str = str(file)
+                            if (
+                                file.suffix == ".py"
+                                and (file.stem == root_module or file.stem == "__init__")
+                            ) or (
+                                "/" in file_str
+                                and (
+                                    file_str.startswith(f"{root_module}/")
+                                    or file_str.startswith(f"{root_module.replace('_', '-')}/")
+                                )
+                            ):
+                                return dist.metadata["Name"]
+                    except Exception:
+                        continue
+            except ImportError:
+                pass
+        except Exception:
+            pass
+
+        return None
+
+    def _extract_third_party_dependencies(
+        self, python_files: list[Path], analyzer: ImportAnalyzer
+    ) -> list[str]:
+        """
+        Extract third-party package dependencies from Python files.
+
+        Analyzes all Python files to find imports classified as "third_party"
+        and returns a list of unique package names. Handles cases where the
+        import name differs from the package name (e.g., 'fitz' -> 'pymupdf').
+
+        Args:
+            python_files: List of Python file paths to analyze
+            analyzer: ImportAnalyzer instance to use for classification
+
+        Returns:
+            List of unique third-party package names (e.g., ["pypdf", "requests", "pymupdf"])
+        """
+        third_party_packages: set[str] = set()
+        # Cache package name lookups to avoid repeated expensive searches
+        package_name_cache: dict[str, str | None] = {}
+
+        total_files = len(python_files)
+        for idx, file_path in enumerate(python_files):
+            if idx > 0 and idx % 50 == 0:
+                print(f"  Analyzing file {idx}/{total_files}...", end="\r", flush=True)
+
+            imports = analyzer.extract_imports(file_path)
+            for imp in imports:
+                analyzer.classify_import(imp, self.src_dir)
+
+                # Extract the root package name (first part of module name)
+                root_module = imp.module_name.split(".")[0]
+
+                # Skip if it's a standard library module
+                stdlib_modules = analyzer.get_stdlib_modules()
+                if root_module in stdlib_modules:
+                    continue
+
+                # Skip if it's local or external (already copied, don't add as dependency)
+                if imp.classification in ("local", "external"):
+                    continue
+
+                # If classified as third_party, try to get actual package name
+                if imp.classification == "third_party":
+                    # Double-check: if it resolves to a file in src_dir, it's actually local
+                    # (might have been copied and now resolves locally)
+                    if imp.resolved_path and imp.resolved_path.is_relative_to(self.src_dir):
+                        continue  # Skip - it's a local file, not a third-party package
+                    # Check cache first
+                    if root_module not in package_name_cache:
+                        package_name_cache[root_module] = self._get_package_name_from_import(
+                            imp.module_name
+                        )
+                    actual_package = package_name_cache[root_module]
+                    if actual_package:
+                        third_party_packages.add(actual_package)
+                    else:
+                        # Fallback to using the import name
+                        third_party_packages.add(root_module)
+                # If it's ambiguous or unresolved, and not stdlib/local/external,
+                # it's likely a third-party package that needs to be declared
+                elif imp.classification == "ambiguous" or imp.classification is None:
+                    # Check if it's not a local or external module
+                    if not imp.resolved_path:
+                        # Check cache first
+                        if root_module not in package_name_cache:
+                            package_name_cache[root_module] = self._get_package_name_from_import(
+                                imp.module_name
+                            )
+                        actual_package = package_name_cache[root_module]
+                        if actual_package:
+                            third_party_packages.add(actual_package)
+                        else:
+                            # Fallback: use import name (will be normalized later)
+                            third_party_packages.add(root_module)
+
+        if total_files > 50:
+            print()  # New line after progress indicator
+
+        return sorted(list(third_party_packages))
+
     def _report_ambiguous_imports(self, python_files: list[Path]) -> None:
         """
         Report any ambiguous imports that couldn't be resolved.
@@ -700,6 +903,11 @@ class BuildManager:
                         publish_package_name = self.subfolder_config.package_name
                     elif package_name:
                         publish_package_name = package_name
+                    else:
+                        # Last resort: derive from src_dir name
+                        publish_package_name = (
+                            self.src_dir.name.replace("_", "-").replace(" ", "-").lower().strip("-")
+                        )
                 else:
                     # For regular builds, get package name from pyproject.toml
                     try:
@@ -718,6 +926,18 @@ class BuildManager:
                                 if "project" in data and "name" in data["project"]:
                                     publish_package_name = data["project"]["name"]
 
+                # Ensure we have package name and version for filtering
+                if is_subfolder_build and not publish_package_name:
+                    raise ValueError(
+                        "Could not determine package name for subfolder build. "
+                        "Please specify --package-name explicitly."
+                    )
+                if is_subfolder_build and not publish_version:
+                    raise ValueError(
+                        "Version is required for subfolder builds. "
+                        "Please specify --version explicitly."
+                    )
+
                 publisher = Publisher(
                     repository=repository,
                     dist_dir=self.project_root / "dist",
